﻿package org.nikas.modules;

import gnu.io.CommPort;
import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;

import java.io.InputStream;
import java.io.OutputStream;

/**
 * Created by Richok 22.01.2014 Ukraine, Dnipropetrovsk (Dneprpetrovsk)
 * Class for controlling the Ke-USB24A module.
 */
public class KeUsb24A {
    private InputStream in;
    private OutputStream out;

    /**
     * Constructor of class receiving name of port which will be opened.
     *
     * @param portName {@link java.lang.String} name of port which will be opened.
     */
    public KeUsb24A(String portName) {
        try {
            open(portName);
        } catch (Exception ex) {
            System.out.println("Не удалось найти устройство Ke-USB24A");
        }
    }

    private int setCommand(String S) throws Exception {
        write(S);
        String s = getAnswer();
        if (s.equals("#ERR\r\n")) {
            return 1;
        } else {
            return 0;
        }
    }

    private void write(String S) throws Exception {
        byte[] b = new byte[100];
        char[] s = S.toCharArray();
        for (int i = 0; i < s.length; i++) {
            b[i] = (byte) s[i];
        }
        out.write(b, 0, s.length);
        byte bb[] = {'\r', '\n'};
        out.write(bb, 0, 2);
    }

    private String getAnswer() throws Exception {
        byte[] buf = new byte[100];
        char[] text = new char[100];
        int i = 0;
        while (buf[0] != '\n') {
            in.read(buf, 0, 1);
            text[i++] = (char) buf[0];
        }
        return new String(text, 0, i);
    }

    private int open(String portName) throws Exception {
        CommPortIdentifier portIdentifier = CommPortIdentifier.getPortIdentifier(portName);
        if (!portIdentifier.isCurrentlyOwned()) {
            return openNotCurrentlyOwnedPort(portIdentifier);
        } else {
            return 1;
        }
    }

    private void setStreams(SerialPort serialPort) throws Exception {
        in = serialPort.getInputStream();
        out = serialPort.getOutputStream();
    }

    private int openNotCurrentlyOwnedPort(CommPortIdentifier portIdentifier) throws Exception {
        CommPort commPort = portIdentifier.open(this.getClass().getName(), 2000);
        if (commPort instanceof SerialPort) {
            SerialPort serialPort = (SerialPort) commPort;
            serialPort.setSerialPortParams(57600, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
            setStreams(serialPort);
            return 0;
        } else {
            return -1;
        }
    }

    /**
     * Function for test the module operability.
     *
     * @return "0" if module works well, else if module doesn't works well.
     */
    public int test() {
        try {
            return setCommand("$KE");
        } catch (Exception ex) {
            return -1;
        }
    }

    /**
     * Function for getting version of module.
     *
     * @return version of module.
     */
    public String getVersion() {
        try {
            if (setCommand("$KE,FW") == 0) {
                write("$KE,FW");
                String s = getAnswer();
                return s.substring(4);
            } else {
                return "FAIL OPERATION";
            }
        } catch (Exception ex) {
            return "FAIL OPERATION";
        }
    }

    /**
     * Function for setting value of the line.
     *
     * @param line  {@link int} number of line which will be set.
     * @param value {@link boolean} value to set.
     * @return "0" if module has done operation, else if module hasn't.
     */
    public int setLineValue(int line, boolean value) {
        try {
            if (value) {
                return setCommand("$KE,WR," + line + "," + 1);
            } else {
                return setCommand("$KE,WR," + line + "," + 0);
            }
        } catch (Exception ex) {
            return -1;
        }
    }

    /**
     * Function for setting value of the line.
     *
     * @param line  {@link int} number of line which will be set.
     * @param value {@link int} value to set.
     * @return "0" if module has done operation, else if module hasn't.
     */
    public int setLineValue(int line, int value) {
        if (value == 0) {
            return setLineValue(line, false);
        } else {
            return setLineValue(line, true);
        }
    }

    /**
     * Function for setting array of values to lines.
     *
     * @param value {@link boolean} array of values for setting.
     * @param count {@link int} count of setting values.
     * @return "0" if module has done operation, else if module hasn't.
     */
    public int setArrayLineValues(boolean[] value, int count) {
        try {
            String s = "$KE,WRA,";
            for (int i = 0; i < value.length; i++) {
                if (value[i]) {
                    s = s + "1";
                } else {
                    s = s + "0";
                }
            }
            return setCommand(s);
        } catch (Exception ex) {
            return -1;
        }
    }

    /**
     * Function for setting values to all 24 lines.
     *
     * @param value {@link boolean} array of values for setting.
     * @return "0" if module has done operation, else if module hasn't.
     */
    public int setArrayLineValues(boolean[] value) {
        return setArrayLineValues(value, 24);
    }

    /**
     * Function for getting value of the line.
     *
     * @param line {@link int} number of line which it needs to read.
     * @return values "0" or "1" according to line status (if module has done operation, else if module hasn't).
     */
    public int getLineValue(int line) {
        try {
            String s = "$KE,RD," + line;
            write(s);
            s = getAnswer();
            if (s.equals("#RD,WRONGLINE\r\n")) {
                return -2;
            } else {
                return Integer.parseInt(s.substring(7, 1));
            }
        } catch (Exception ex) {
            return -1;
        }
    }

    /**
     * Function for getting values of all 24 lines.
     *
     * @return array of values "0" or "1" according to lines' status (if module has done operation,
     * else if module hasn't).
     */
    public int[] getLineValue() {
        int[] res = new int[24];
        try {
            String ss, s;
            write("$KE,RD,ALL");
            s = getAnswer();
            s = s.substring(4, 28);
            for (int i = 0; i < 24; i++) {
                ss = s.substring(i, i + 1);
                if (ss.equals("x")) {
                    res[i] = -1;
                } else {
                    res[i] = Integer.parseInt(s.substring(i, 1));
                }
            }
            return res;
        } catch (Exception ex) {
            return res;
        }
    }

    /**
     * Function for getting ADC result with determined frequency.
     *
     * @param frequency {@link int} frequency of ADC.
     * @return result of readings.
     */
    public int getAdcResult(int frequency) {
        try {
            write("$KE,ADC," + frequency);
            String s = getAnswer();
            s = s.substring(5, s.length() - 2);
            int res = Integer.parseInt(s);
            return res;
        } catch (Exception ex) {
            return -1;
        }
    }
}